
%{
/*
 * Copyright
 * =========
 * This software is copyright P.J.H.Fox (fox@roestock.demon.co.uk)
 * and distributed under the GPL (see COPYING).
 * 
 * If mailing patches, bug reports or comments, please put 'fpath'
 * somewhere in the subject line, and mail to fpath@roestock.demon.co.uk.
 */

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <time.h>
#include <linux/limits.h> /* For PATH_MAX */

static char rcsid[] = "$Id: fpath.sun.y,v 1.2 1996/02/11 09:57:15 fox Exp $";

#define SIZE_MAX 16

char path[PATH_MAX] = "";
char size[SIZE_MAX];
int month;
int day;
int year;
char mode;

int todayyear, todaymonth;
int yywrap () { return 1;}

%}

/*
total 30
-rw-r--r--   1 fox      system      17982 Feb  8 22:15 COPYING
-rw-r--r--   1 fox      system        368 Feb  8 22:56 FILES
-rw-r--r--   1 fox      system        224 Feb  8 22:56 Makefile
drwxr-xr-x   2 fox      system       1024 Feb 10 08:58 RCS
-rw-r--r--   1 fox      system       1510 Feb  8 22:56 README
-rw-r--r--   1 fox      system       4328 Feb 10 09:18 fpath.y
drwxr-xr-x   3 fox      system       1024 Feb 10 09:19 test

RCS:
total 0

test:
total 1
lrwxrwxrwx   1 fox      system         11 Feb 10 09:19 bloggs -> subdir/fred
drwxr-xr-x   2 fox      system       1024 Feb 10 09:19 subdir

test/subdir:
total 0
-rw-r--r--   1 fox      system          0 Feb 10 09:19 fred

total 27
crw-r--r--   1 root     daemon    10,   0 Oct 27  1992 bm
lrwxrwxrwx   1 root     system          8 Jan  2 20:09 cdrom -> /dev/hdc
brw-r--r--   1 root     daemon    11,   1 Dec 14  1992 cdrom1
brw-r--r--   1 root     daemon    13,  72 Jul 21  1993 xdb8
crw-r--r--   1 root     daemon     1,   5 Jul 21  1993 zero
*/

%s SIZE
%s DEVSIZE
%s MONTH
%s DAY
%s YEARTIME
%s NAME

%%

<INITIAL>[^ \t\n]*:	{  strncpy(path, yytext, PATH_MAX-1); path[yyleng - 1] = '/'; }
<INITIAL>[\-l][\-drwsxt]{9}[ \t]*[^ \t]*[ \t]*[^ \t]*[ \t]*	{  BEGIN(SIZE); mode = yytext[0]; if(mode == '-') mode = yytext[3]; }
<INITIAL>[cb][\-drwsxt]{9}[ \t]*[^ \t]*[ \t]*[^ \t]*[ \t]*	{  BEGIN(DEVSIZE); mode = yytext[0];}
<INITIAL>.	{ /* Eat anything else to eol */ register int c;  while(((c = input()) != '\n') && (c != EOF)) ; }

<SIZE>[^ \t]* {  BEGIN(MONTH); strncpy(size, yytext, SIZE_MAX-1); }

<DEVSIZE>[^ \t]*,[ \t]*[^ \t]* {  BEGIN(MONTH); strncpy(size, yytext, SIZE_MAX-1); }

<MONTH>"Jan" {  BEGIN(DAY); month = 1; }
<MONTH>"Feb" {  BEGIN(DAY); month = 2; }
<MONTH>"Mar" {  BEGIN(DAY); month = 3; }
<MONTH>"Apr" {  BEGIN(DAY); month = 4; }
<MONTH>"May" {  BEGIN(DAY); month = 5; }
<MONTH>"Jun" {  BEGIN(DAY); month = 6; }
<MONTH>"Jul" {  BEGIN(DAY); month = 7; }
<MONTH>"Aug" {  BEGIN(DAY); month = 8; }
<MONTH>"Sep" {  BEGIN(DAY); month = 9; }
<MONTH>"Oct" {  BEGIN(DAY); month = 10; }
<MONTH>"Nov" {  BEGIN(DAY); month = 11; }
<MONTH>"Dec" {  BEGIN(DAY); month = 12; }
<MONTH>"."   {  BEGIN(INITIAL); fprintf(stderr, "Duff char '%s' looking for month\n", yytext); }

<DAY>[^ \t]* {  BEGIN(YEARTIME); day = atol(yytext); }

<YEARTIME>[0-9]{4}[ \t]* {  BEGIN(NAME); year = atol(yytext); }
<YEARTIME>[0-9]+:[0-9]+[ \t]* {  BEGIN(NAME); year = todayyear; if(month > todaymonth) year--; }

<NAME>[^ \t\n]*"/"	fprintf(stderr, "Dodgy ls -lR --- path component '%s' in file name\n", yytext); /* delete any paths in listing (No possible !) */
<NAME>[^/\n]*(" -> "[^\n]*)?$ {  BEGIN(INITIAL); printf("%04d/%02d/%02d %10s %c %s%s\n", year, month, day, size, mode, path, yytext);}

[ \t\n]*	/* Swallow whitespace */
.	fprintf(stdout, "Don't know what to do with %s\n", yytext); 

%%

int
main(int argc, char **argv)
{
	int retval;
	time_t todaysecs;
	struct tm *today;
	struct timeval tv;
	struct timezone tz;
	if(argc > 2)
	{
		fprintf(stderr, "%s\nThis is free software, see the sources for details.\nThis software has NO WARRANTY, use at your OWN RISK\n\nUsage: ls -lR [/abspath] | %s [/abspath]\n", rcsid, argv[0]);
		exit(1);
	}
	if(argc == 2)
	{
		strncpy(path, argv[1], PATH_MAX-2);
		if(path[strlen(path)-1] != '/')
			strcat(path, "/");
	}
	retval = 0;
	gettimeofday(&tv, &tz);
	todaysecs = tv.tv_sec;
	today = gmtime(&todaysecs);
	todayyear = today->tm_year;
	if(todayyear < 1900)
		todayyear += 1900;
	todaymonth = today->tm_mon + 1;
	yyin = stdin;
	yylex();
	return retval;
}

